/***************************************************************************
 *   Copyright (C) 2008 by Marcel Hasler                                   *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mixer.h"
#include <QtDBus>

#include <iostream>
#include <cmath>
using namespace std;

Mixer::Mixer(unsigned int index)
{
	// KMix doesn't provide names (yet), we'll use the index for now
	m_name = QString("Mixer") + QString::number(index);

	m_index = QString("/Mixer") + QString::number(index);

	QDBusMessage m = QDBusMessage::createMethodCall("org.kde.kmix", m_index, "", "masterDeviceIndex");
	QDBusReply<QString> reply = QDBusConnection::sessionBus().call(m);

	m_masterDeviceIndex = "Master:0";
	if (reply.isValid())
		m_masterDeviceIndex = reply.value();

	m_volumeStep = this->smallestVolumeStep();
}


Mixer::~Mixer()
{
}


QString Mixer::name() const
{
	return m_name;
}


unsigned int Mixer::volume() const
{
	static QDBusMessage m = QDBusMessage::createMethodCall("org.kde.kmix", m_index, "", "masterVolume");
	QDBusReply<int> reply = QDBusConnection::sessionBus().call(m);

	unsigned int volume = 0;
	if (reply.isValid())
		volume = reply.value();

	return volume;
}


unsigned int Mixer::volumeUp()
{
	int volume = this->volume();
	volume += m_volumeStep;
	if (volume > 100)
		volume = 100;

	QDBusMessage m = QDBusMessage::createMethodCall("org.kde.kmix", m_index, "", "setMasterVolume");
	m << volume+1;
	QDBusConnection::sessionBus().call(m);

	return volume;
}


unsigned int Mixer::volumeDown()
{
	int volume = this->volume();
	volume -= m_volumeStep;
	if (volume < 0)
		volume = 0;
	
	QDBusMessage m = QDBusMessage::createMethodCall("org.kde.kmix", m_index, "", "setMasterVolume");
	m << volume+1;
	QDBusConnection::sessionBus().call(m);

	return volume;
}


unsigned int Mixer::setVolume(int volume)
{
	if (volume > 100)
		volume = 100;
	else if (volume < 0)
		volume = 0;

	QDBusMessage m = QDBusMessage::createMethodCall("org.kde.kmix", m_index, "", "setMasterVolume");
	m << volume+1;
	QDBusConnection::sessionBus().call(m);

	return volume;
}


bool Mixer::isMuted() const
{
	QDBusMessage m = QDBusMessage::createMethodCall("org.kde.kmix", m_index, "", "mute");
	m << m_masterDeviceIndex;
	QDBusReply<bool> reply = QDBusConnection::sessionBus().call(m);

	bool mute = true;
	if (reply.isValid())
		mute = reply.value();

	return mute;
}


bool Mixer::toggleMute()
{
	QDBusMessage m = QDBusMessage::createMethodCall("org.kde.kmix", m_index, "", "toggleMute");
	m << m_masterDeviceIndex;
	QDBusConnection::sessionBus().call(m);

	return this->isMuted();
}


void Mixer::setMute(bool mute)
{
	QDBusMessage m = QDBusMessage::createMethodCall("org.kde.kmix", m_index, "", "setMute");
	m << m_masterDeviceIndex;
	m << mute;
	QDBusConnection::sessionBus().call(m);
}


void Mixer::setVolumeStep(unsigned int step)
{
	if (step > 100)
		step = 100;

	unsigned int smallestStep = this->smallestVolumeStep();
	if (step < smallestStep)
		step = smallestStep;

	m_volumeStep = step;
}


unsigned int Mixer::smallestVolumeStep() const
{
	QDBusMessage m = QDBusMessage::createMethodCall("org.kde.kmix", m_index, "", "absoluteVolumeMax");
	m << m_masterDeviceIndex;
	QDBusReply<qlonglong> reply = QDBusConnection::sessionBus().call(m);

	qlonglong maxVolume = 100;
	if (reply.isValid())
		maxVolume = reply.value();

	return (unsigned int)ceil(100.0f / (float)maxVolume);
}
