/***************************************************************************
 *   Copyright (C) 2008-2009 by Marcel Hasler                              *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "powermanager.h"

#include <Solid/Device>
#include <Solid/DeviceInterface>
#include <solid/control/powermanager.h>

PowerManager::PowerManager()
	: m_hasBattery(false)
{
	QList<Solid::Device> batteries = Solid::Device::listFromType(Solid::DeviceInterface::Battery);
	if (batteries.size() >= 1)
	{
		m_hasBattery = true;
		m_battery = batteries[0].as<Solid::Battery>();
	}
}


PowerManager::~PowerManager()
{

}


int PowerManager::brightness() const
{
	return (int)Solid::Control::PowerManager::brightness();
}


int PowerManager::setBrightness(int percent)
{
	Solid::Control::PowerManager::setBrightness((float)percent);
	return this->brightness();
}


int PowerManager::batteryCharge() const
{
	if (m_hasBattery)
		return m_battery->chargePercent();
	else
		return -1;
}


PowerManager::BatteryState PowerManager::batteryState() const
{
	if (m_battery->chargeState() == Solid::Battery::NoCharge)
		return NoCharge;
	if (m_battery->chargeState() == Solid::Battery::Charging)
		return Charging;
	if (m_battery->chargeState() == Solid::Battery::Discharging)
		return Discharging;

	return Error;
}
